﻿/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using moxa;

namespace SNTP
{
    class Program
    {
        private static uint CTL_CODE(uint DeviceType, uint Function, uint Method, uint Access)
        {
            return ((DeviceType << 16) | (Access << 14) | (Function << 2) | Method);
        }

        #region Windows CE API Constant
        public readonly static IntPtr INVALID_HANDLE_VALUE = new IntPtr(-1);
        public const UInt32 OPEN_EXISTING = 3;
        public const UInt32 GENERIC_READ = 0x80000000;
        public const UInt32 GENERIC_WRITE = 0x40000000;
        #endregion

        #region Device IO Control Code
        public const UInt32 FILE_DEVICE_HAL = 0x00000101;
        public const UInt32 FILE_DEVICE_CONSOLE = 0x00000102;
        public const UInt32 FILE_DEVICE_PSL = 0x00000103;
        public const UInt32 FILE_DEVICE_SERVICE = 0x00000104;
        public const UInt32 FILE_DEVICE_NLED = 0x00000105;
        public const UInt32 FILE_DEVICE_NOTIFY = 0x00000106;

        public const UInt32 METHOD_BUFFERED = 0;
        public const UInt32 METHOD_IN_DIRECT = 1;
        public const UInt32 METHOD_OUT_DIRECT = 2;
        public const UInt32 METHOD_NEITHER = 3;

        public const UInt32 FILE_ANY_ACCESS = 0;
        public const UInt32 FILE_READ_ACCESS = (0x0001);    // file & pipe
        public const UInt32 FILE_WRITE_ACCESS = (0x0002);    // file & pipe

        public static uint IOCTL_GPIO_BEEP_ON = 106 << 2;

        public static uint IOCTL_RTC_SET = CTL_CODE(0, 42, 0, 0);
        public static uint IOCTL_RTC_GET = CTL_CODE(0, 43, 0, 0);

        public static uint IOCTL_START_WDG = CTL_CODE(0, 164, 0, 0);
        public static uint IOCTL_STOP_WDG = CTL_CODE(0, 165, 0, 0);
        public static uint IOCTL_REFRESH_WDG = CTL_CODE(0, 166, 0, 0);

        public static uint IOCTL_BUZZER_ON = CTL_CODE(0, 167, 0, 0);
        public static uint IOCTL_BUZZER_OFF = CTL_CODE(0, 168, 0, 0);

        public static uint IOCTL_HAL_REBOOT = CTL_CODE(FILE_DEVICE_HAL, 15, METHOD_BUFFERED, FILE_ANY_ACCESS);

        public static uint IOCTL_SERVICE_CONTROL = CTL_CODE(FILE_DEVICE_SERVICE, 7, METHOD_BUFFERED, FILE_ANY_ACCESS);

        public const uint MOXA_SET_OP_MODE = (0x400 + 66);
        public const uint MOXA_GET_OP_MODE = (0x400 + 67);
        public const uint RS232_MODE = 0;
        public const uint RS485_2WIRE_MODE = 1;
        public const uint RS422_MODE = 2;
        public const uint RS485_4WIRE_MODE = 3;

        #endregion

        public static uint IOCTL_SYNC_NTP = CTL_CODE(0, 167, 0, 0);

        static void Main(string[] args)
        {
            //create the service object
            moxa.mxdevice dev = new moxa.mxdevice("UCS1:");

            //make sure  the object is opened
            if (dev.Open())
            {
                string szSync = "sync";
                int nBytesReturn = 0;
                int nSize = szSync.Length;

                IntPtr lpstr = Marshal.AllocHGlobal(nSize);

                //sync with ntp server             
                dev.DeviceIoControl(IOCTL_SERVICE_CONTROL, lpstr, nSize, IntPtr.Zero, 0, ref nBytesReturn, IntPtr.Zero);

                Marshal.FreeHGlobal(lpstr);

                //close the object
                dev.Close();
            }

        }
    }
}


