// MessageQueue.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>

#define QUEUE_NAME L"MQ_DEMO"

struct _DATA_t {
	unsigned int id;
	BYTE *pData;
	DWORD dwLen;
};

BOOL bQuit = FALSE;
CRITICAL_SECTION cs;

int getRand(int nID);

#define CLIENTS 5

DWORD thread_main(LPVOID PPARAM)
{
	MSGQUEUEOPTIONS msq;
	_DATA_t m_data;
	DWORD dwRead, dwflags;
	DWORD dwDataRecvCount[CLIENTS];
	DWORD dwDataRecvBytes[CLIENTS];
	
	for ( int i=0; i<CLIENTS; i++) {
		dwDataRecvCount[i] = 0;
		dwDataRecvBytes[i] = 0;
	}

	msq.dwSize = sizeof(MSGQUEUEOPTIONS);
	msq.bReadAccess = TRUE;                  // Read permission
	msq.dwMaxMessages = 0;                   // Set the max messages number of message queue to be INFINITE
	msq.dwFlags =  MSGQUEUE_NOPRECOMMIT | MSGQUEUE_ALLOW_BROKEN;
	msq.cbMaxMessage = sizeof(_DATA_t);      // assign to message data structure size

	// create a read only message queue
	HANDLE hMsq = CreateMsgQueue(QUEUE_NAME, &msq);
	printf("Press [Enter] to leave program!! ...\n");

	while (TRUE)
	{
		// waiting for data goes into the Message Queue
		switch ( WaitForSingleObject(hMsq, 500) )
		{
			case WAIT_OBJECT_0:
				// read data from message queue
				if ( ReadMsgQueue( hMsq, &m_data, sizeof(m_data), &dwRead, INFINITE, &dwflags) ) {
					dwDataRecvCount[m_data.id-1]++;
					dwDataRecvBytes[m_data.id-1] += m_data.dwLen;
				}
				printf("\r");
				for ( int j=0; j<CLIENTS; j++) {
					printf("ID%d=%3d(%5d) ", j+1, dwDataRecvCount[j], dwDataRecvBytes[j]);
				}
				break;
			case WAIT_TIMEOUT:
				break;
			case WAIT_FAILED:
				printf("WAIT_FAILED[%d]\n", GetLastError());
				break;
			default:
				break;
		}
		if (bQuit)
			break;
	}

	// close message queue
	CloseMsgQueue(hMsq);

	return 0;
}

DWORD thread_collectdata(LPVOID pnIndex)
{
	int nID = *(int*)pnIndex;
	MSGQUEUEOPTIONS msq;

	msq.dwSize = sizeof(MSGQUEUEOPTIONS);
	msq.bReadAccess = FALSE;                    // Write permission
	msq.dwMaxMessages = 0;                      // Set the max messages number of message queue to be INFINITE
	msq.dwFlags =  MSGQUEUE_NOPRECOMMIT | MSGQUEUE_ALLOW_BROKEN;
	msq.cbMaxMessage = sizeof(_DATA_t);         // assign to message data structure size

	// create a write only message queue
	HANDLE hTask = CreateMsgQueue(QUEUE_NAME, &msq);

	while (TRUE)
	{
		_DATA_t data;
		data.id = nID;                     // Write ID to identify who I am
		data.pData = NULL;                 // not use
		data.dwLen = getRand(nID);         // get random number

		// write data into message queue
		WriteMsgQueue( hTask, &data, sizeof(data), INFINITE, 0);

		// sleep few times
		Sleep( nID * 200 );

		if (bQuit)
			break;
	}

	// close message queue
	CloseMsgQueue(hTask);

	return 0;
}

int getRand(int nID)
{
	EnterCriticalSection(&cs);

    /* Seed the random-number generator with GetTickCount so that the numbers will be different every time we run. */
    srand( GetTickCount() );

    /* get a number between 0 and 499 */
	int nResult = rand() % 500;

	LeaveCriticalSection(&cs);

    return nResult;
}


int _tmain(int argc, _TCHAR* argv[])
{
	DWORD dwData[CLIENTS];

	// Initializing the critical section variable
	InitializeCriticalSection(&cs);

	// create the main thread first for waiting data
	CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)thread_main, NULL,0, NULL);
	Sleep(500);

	for ( int i=0; i<CLIENTS; i++) {
		dwData[i] = i+1;
		// create the client thread
		CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)thread_collectdata, &dwData[i],0, NULL);
	}

	// wait a command to close all threads
	WCHAR ch = getwchar();
	bQuit = TRUE;
	DeleteCriticalSection(&cs);

	// make sure all threads were closed
	Sleep(3000);

	return 0;
}

