/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include <Iphlpapi.h>
#include <pm.h>
#include "fsioctl.h"

#define DD_NDIS_DEVICE_NAME TEXT("NDS0:")

#define _NDIS_CONTROL_CODE(request,method) \
            CTL_CODE(FILE_DEVICE_PHYSICAL_NETCARD, request, method, FILE_ANY_ACCESS)

#define WINCE_IOCTL_START 8
#define	IOCTL_NDIS_POWER_RESUME			_NDIS_CONTROL_CODE( WINCE_IOCTL_START+0, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_REGISTER_ADAPTER		_NDIS_CONTROL_CODE( WINCE_IOCTL_START+1, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_DEREGISTER_ADAPTER	_NDIS_CONTROL_CODE( WINCE_IOCTL_START+2, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_REBIND_ADAPTER		_NDIS_CONTROL_CODE( WINCE_IOCTL_START+3, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_BIND_ADAPTER			_NDIS_CONTROL_CODE( WINCE_IOCTL_START+4, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_UNBIND_ADAPTER		_NDIS_CONTROL_CODE( WINCE_IOCTL_START+5, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_GET_ADAPTER_NAMES	_NDIS_CONTROL_CODE( WINCE_IOCTL_START+6, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_GET_PROTOCOL_NAMES	_NDIS_CONTROL_CODE( WINCE_IOCTL_START+7, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_GET_ADAPTER_BINDINGS	_NDIS_CONTROL_CODE( WINCE_IOCTL_START+8, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_LOAD_MINIPORT		_NDIS_CONTROL_CODE( WINCE_IOCTL_START+9, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_GET_VERSION			_NDIS_CONTROL_CODE( WINCE_IOCTL_START+10, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_LOAD_RETAIL_LOGGER	_NDIS_CONTROL_CODE( WINCE_IOCTL_START+11, METHOD_OUT_DIRECT )
#define	IOCTL_NDIS_UNLOAD_RETAIL_LOGGER _NDIS_CONTROL_CODE( WINCE_IOCTL_START+12, METHOD_OUT_DIRECT )

#define NDISPWR_DEVICE_NAME     TEXT("NPW1:")

#define REG_NDISPOWER_ROOT      TEXT("Comm\\NdisPower")

#define FSCTL_NDISPWR_BASE      FILE_DEVICE_NETWORK

#define _NDISPWR_CTL_CODE(_Function, _Method, _Access)  \
            CTL_CODE(FSCTL_NDISPWR_BASE, _Function, _Method, _Access)


//
//  Valid IOCTL for NPW1:
//

#define IOCTL_NPW_SAVE_POWER_STATE  \
            _NDISPWR_CTL_CODE(0x200, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_NPW_QUERY_SAVED_POWER_STATE  \
            _NDISPWR_CTL_CODE(0x201, METHOD_BUFFERED, FILE_ANY_ACCESS)            


typedef struct _NDISPWR_SAVEPOWERSTATE
{
    LPCWSTR                pwcAdapterName;
    CEDEVICE_POWER_STATE   CePowerState;

}   NDISPWR_SAVEPOWERSTATE, *PNDISPWR_SAVEPOWERSTATE;

BOOL DoNdisIOControl(DWORD dwCommand, LPVOID pInBuffer,
                                  DWORD cbInBuffer, LPVOID pOutBuffer,
                                  DWORD * pcbOutBuffer)
{
   HANDLE hNdis;
   BOOL fResult = FALSE;

   hNdis = ::CreateFile(DD_NDIS_DEVICE_NAME, GENERIC_READ | GENERIC_WRITE,
                       FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_ALWAYS,
                        0, NULL);

   if (INVALID_HANDLE_VALUE != hNdis)
   {
      fResult = ::DeviceIoControl(hNdis, dwCommand, pInBuffer, cbInBuffer,
                                  NULL,0,
                                  pcbOutBuffer, NULL);
	  if(fResult)
	  {
		  printf("DeviceIoControl Success!\n");
	  }
	  else
	  {
		  printf("DeviceIoControl Fail!\n");
	  }
      ::CloseHandle(hNdis);
   }
   else
   {
	   printf("CreateFile Fail!");
   }

   return fResult;
}

int _tmain(int argc, _TCHAR* argv[])
{
	//Enable the power state of this adapter
	HANDLE                  hNdisPwr;
	NDISPWR_SAVEPOWERSTATE  SavePowerState;
	WCHAR  *szLAN1 = L"PCI\\E100CE1\0";
	WCHAR  *szLAN2 = L"PCI\\RTCENIC1\0";

	hNdisPwr = CreateFile(
    		    (PTCHAR)NDISPWR_DEVICE_NAME,					//	Object name.
    		    0x00,											//	Desired access.
    		    0x00,											//	Share Mode.
    		    NULL,											//	Security Attr
    		    OPEN_EXISTING,									//	Creation Disposition.
    		    FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,	//	Flag and Attributes..
    		    (HANDLE)INVALID_HANDLE_VALUE);	
     if (hNdisPwr != INVALID_HANDLE_VALUE)
	 {
        SavePowerState.pwcAdapterName = szLAN1;                 // Specify the Lan1 to enable
		SavePowerState.CePowerState   = PwrDeviceUnspecified;
		printf("adapter power on!\n");
	 }

	 DeviceIoControl(
            hNdisPwr,
            IOCTL_NPW_SAVE_POWER_STATE,
            &SavePowerState,
            sizeof(NDISPWR_SAVEPOWERSTATE),
            NULL,
            0x00,
            NULL,
            NULL);    
	 CloseHandle(hNdisPwr);
	
	//Enable Lan Connection
	DoNdisIOControl(
            IOCTL_NDIS_BIND_ADAPTER, 
            szLAN1,                                            // rebind LAN1
            (wcslen(szLAN1)+1) * 2,
            NULL, 
            NULL);   

	return 0;
}

